--*** SYNTHETIC PIC V1.1 ***
--
-- VHDL
--
-- Entity:	REGFILE
--	Purpose: This is a synthesizable Register File
--
--      Special Notes:
--           For the PIC, registers 0-7 are special registers
--           and are not part of this module; they are the special
--           PIC registers like the PC and STATUS register.
--
-- Note: This entity is most likely going to be synthesized in some other way
-- for a real ASIC or even FPGA (although it is synthesizable as is).
-- Some sort of MEMGEN program specific to the technology being used
-- can be used.
--
-- See licencing agreement in the file PICCPU.VHD
--

-- VIEWLOGIC libraries suitable for both simulation and for synthesis.
--
library ieee;
use ieee.std_logic_1164.ALL;
use ieee.std_logic_unsigned.all;

entity PICREGS is
  port (
	 Clk		:	in  std_logic;

	 WriteEnable    :       in std_logic;

	 -- Interface to microprogram in ROM
	 FSel    :  in std_logic_vector(4 downto 0);

	 -- Input busses
	 Din    :  in  std_logic_vector(7 downto 0);

	 -- Output busses
	 Dout    :  out std_logic_vector(7 downto 0));
end PICREGS;


architecture first of PICREGS is

-- Registers
--
-- Note: I've commented out the higher registers beyond R12 because I
--       did not reference them in my test, and they bog down my
--       synthesizer.
--
signal R8  :  std_logic_vector (7 downto 0);
signal R9  :  std_logic_vector (7 downto 0);
signal R10 :  std_logic_vector (7 downto 0);
signal R11 :  std_logic_vector (7 downto 0);
signal R12 :  std_logic_vector (7 downto 0);
-- ** Let's not synthesize any more for now..
-- signal R13 :  std_logic_vector (7 downto 0);
-- signal R14 :  std_logic_vector (7 downto 0);
-- signal R15 :  std_logic_vector (7 downto 0);
-- signal R16 :  std_logic_vector (7 downto 0);
-- signal R17 :  std_logic_vector (7 downto 0);
-- signal R18 :  std_logic_vector (7 downto 0);
-- signal R19 :  std_logic_vector (7 downto 0);
-- signal R20 :  std_logic_vector (7 downto 0);
-- signal R21 :  std_logic_vector (7 downto 0);
-- signal R22 :  std_logic_vector (7 downto 0);
-- signal R23 :  std_logic_vector (7 downto 0);
-- signal R24 :  std_logic_vector (7 downto 0);
-- signal R25 :  std_logic_vector (7 downto 0);
-- signal R26 :  std_logic_vector (7 downto 0);
-- signal R27 :  std_logic_vector (7 downto 0);
-- signal R28 :  std_logic_vector (7 downto 0);
-- signal R29 :  std_logic_vector (7 downto 0);
-- signal R30 :  std_logic_vector (7 downto 0);
-- signal R31 :  std_logic_vector (7 downto 0);

begin
	-- Concurrently always route register outputs out the register file based
	-- on the select lines.
	--
	Dout <= R8  When conv_integer(Fsel) =  8 Else
			  R9  When conv_integer(Fsel) =  9 Else
			  R10 When conv_integer(Fsel) = 10 Else
			  R11 When conv_integer(Fsel) = 11 Else
			  R12 When conv_integer(Fsel) = 12 Else
			  -- R13 When conv_integer(Fsel) = 13 Else
			  -- R14 When conv_integer(Fsel) = 14 Else
			  -- R15 When conv_integer(Fsel) = 15 Else
			  -- R16 When conv_integer(Fsel) = 16 Else
			  -- R17 When conv_integer(Fsel) = 17 Else
			  -- R18 When conv_integer(Fsel) = 18 Else
			  -- R19 When conv_integer(Fsel) = 19 Else
			  -- R20 When conv_integer(Fsel) = 20 Else
			  -- R21 When conv_integer(Fsel) = 21 Else
			  -- R22 When conv_integer(Fsel) = 22 Else
			  -- R23 When conv_integer(Fsel) = 23 Else
			  -- R24 When conv_integer(Fsel) = 24 Else
			  -- R25 When conv_integer(Fsel) = 25 Else
			  -- R26 When conv_integer(Fsel) = 26 Else
			  -- R27 When conv_integer(Fsel) = 27 Else
			  -- R28 When conv_integer(Fsel) = 28 Else
			  -- R29 When conv_integer(Fsel) = 29 Else
			  -- R30 When conv_integer(Fsel) = 30 Else
			  -- R31 When conv_integer(Fsel) = 31 Else
			  "00000000";

	-- Writes are synchronous with the clk.
	--
	DoIt: process (clk)
	begin
	if rising_edge(Clk) then
		if WriteEnable = '1' then
			-- Write Din into the selected register
		  case conv_integer(FSel) is
			 When 8  => R8  <= Din;
			 When 9  => R9  <= Din;
			 When 10 => R10 <= Din;
			 When 11 => R11 <= Din;
			 When 12 => R12 <= Din;
			 -- When 13 => R13 <= Din;
			 -- When 14 => R14 <= Din;
			 -- When 15 => R15 <= Din;
			 -- When 16 => R16 <= Din;
			 -- When 17 => R17 <= Din;
			 -- When 18 => R18 <= Din;
			 -- When 19 => R19 <= Din;
			 -- When 20 => R20 <= Din;
			 -- When 21 => R21 <= Din;
			 -- When 22 => R22 <= Din;
			 -- When 23 => R23 <= Din;
			 -- When 24 => R24 <= Din;
			 -- When 25 => R25 <= Din;
			 -- When 26 => R26 <= Din;
			 -- When 27 => R27 <= Din;
			 -- When 28 => R28 <= Din;
			 -- When 29 => R29 <= Din;
			 -- When 30 => R30 <= Din;
			 -- When 31 => R31 <= Din;
			 when OTHERS => NULL;
		  end case;
		end if;
	end if;
	end process DoIt;
end first;
